Attribute VB_Name = "basCalView"
Option Explicit
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        All the global functions
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
'Declarations
Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" _
    (lpvDest As Any, lpvSource As Any, ByVal cbCopy As Long)

Private Const msMODULE_NAME      As String = "basCalView"

' Registry Path for Calendar Categories
Private Const CATEGORY_SEARCH_PATH = "Software\Microsoft\Office\8.0\Outlook\Categories"

' Const's used for addbar function cuz it was getting confusing
Private Const BAR_SIZE = 50
Private Const BAR_OFFSET = 25
Private Const BAR_PATTERN = 0               ' No pattern - just solid bar
Private Const BAR_ADJUSTABLE = 4            ' Bar can NOT be moved
Private Const BAR_TEXT_POSITION = 5         ' Where does appt text start (left justified)
Private Const BarTextClip = True            ' Adjust width of text to width of bar
Private Const BarTextField = 10             ' Which SchedOCX field to use for appt bar text
Private Const BarBegSymbol = 0              ' No start symbol on bar
Private Const BarBegSize = 50
Private Const BarBegOffset = 25
Private Const BarEndSymbol = 0              ' No end symbol on bar
Private Const BarEndtype = 0
Private Const BarEndSize = 50
Private Const BarEndOffset = 25

Private Const MAXIMUM_NAME_BARS = 1024
Private Const MAXIMUM_APPOINTMENTS = 32000

Private Const ProfileBufferSize = 2000      ' Max string buffer for get/write private profile

Public golApplication As Outlook.Application
Public golNamespace As Outlook.NameSpace
Public gbScheduleUpdated As Boolean
Public gbCurrentNamesSaved As Boolean

Public gsProfileName As String
Public gsResults As String

Public gsWindowsUserName As String
Public gsCurrentNameList As String
Public gsDefaultNameList As String

Public gcolColors As Collection

Private gasCategoriesToInclude() As String

Private mcolLocationColors As New Collection

Private Sub ClearLocationColors()

  With mcolLocationColors
    Do While .Count > 0
      .Remove .Count
    Loop
  End With

End Sub

Public Function ConvertMapiError(lngErrNumber As Long)
    ConvertMapiError = LoWord(lngErrNumber) + 1000
End Function

Private Function LoWord(dw As Long) As Integer
    Static w As Integer
    CopyMemory w, dw, 2
    LoWord = w
End Function

Public Sub DisplayRebuildCancelNotice()

Dim sMsg As String

  sMsg = "Update cancelled.  Schedules might be incomplete."
  gsResults = gsResults & " - Cancelled" & vbCrLf

  MsgBox sMsg, vbInformation, App.Title & " " & App.Major & "." & App.Minor
End Sub

Public Sub GetAppointments(oNames As clsResources, sCriteria As String, bAll As Boolean, bProgressBar As Boolean)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose
'Inputs
'Assumes
'Effects
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "GetAppointMents"    ' Error logging

Dim olRecipient As Outlook.Recipient
Dim olMAPIFolder As Outlook.MAPIFolder

Dim nBarColor As Integer        ' What color is the next bar?
Dim nBarFontColor As Integer    ' What color is the font for the next bar?
Dim nLBCount As Integer
Dim oResource As clsResource

Dim nResult As Integer
Dim nTBIndex As Integer
Dim sErrorDesc As String
Dim nPBNameCounter As Integer
Dim bSkip As Boolean            ' True = Skip this record

  On Error GoTo GetAppointMents_Error
  
  nTBIndex = 0
      
  ' Set resource & bar array size
  ' Each name is entered into the resource array and into the timebar array
  ' timebars are a 1 to 1 relationship and array limit is MAXIMUMNAMEBARS (16384)
  If oNames.Count <= MAXIMUM_NAME_BARS Then
    frmCalView.schAppointments.ResMax = oNames.Count
  Else
    MsgBox App.Title & " is limited to a total of " & MAXIMUM_NAME_BARS & " Names", vbCritical
    frmCalView.schAppointments.ResMax = MAXIMUM_NAME_BARS
  End If
  
  ' Here we create 16 color bars - a task will refer to one of these
  ' bars when created - so we can color by person or by location
  frmCalView.schAppointments.BarsMax = 16
  For nBarColor = 0 To 15
    ' Define bar appearance
    If nBarColor > 6 Then
      nBarFontColor = 0
    Else
      nBarFontColor = 15
    End If
    ' Add bar to control
    nResult = frmCalView.schAppointments.AddBar(nBarColor, "Bar" & nBarColor, BAR_SIZE, BAR_OFFSET, QBColor(nBarColor), BAR_PATTERN, BAR_ADJUSTABLE, BAR_TEXT_POSITION, BarTextField, "", BarTextClip, QBColor(nBarFontColor), "Arial", 100, False, False, BarBegSymbol, 0, QBColor(0), BarBegSize, BarBegOffset, BarEndSymbol, BarEndtype, QBColor(0), BarEndSize, BarEndOffset)
  Next nBarColor
  
  ' Clear update results list box
  gsResults = "---> Last Updated " & Now & " <---" & vbCrLf
    
  ' This FOR loop will process the resources alphabetically (Listbox is sorted)
  For nLBCount = 0 To oNames.ResourceListBox.ListCount - 1
    ' Resolve the listbox index to a particular resource object (Key stored in ItemData)
    Set oResource = oNames.Item("K" & oNames.ResourceListBox.ItemData(nLBCount))
    
    DoEvents
    If frmProgressBar.Cancel Then
      Call DisplayRebuildCancelNotice
      GoTo GetAppointMents_Exit
    End If
        
    bSkip = False
    If bAll Or oResource.Selected Then
        
      ' Display current name
      gsResults = gsResults & oResource.FullName
                  
      ' Increment Progressbar value
      nPBNameCounter = nPBNameCounter + 1
      ' Create an Outlook recipient object from current name
      Set olRecipient = golNamespace.CreateRecipient(oResource.FullName)
      
      ' resolve name name from listbox against Outook addressbooks
      olRecipient.Resolve
      
      If Not olRecipient.Resolved Then
        ' If the full name did not resolve, then try the alias
        Set olRecipient = golNamespace.CreateRecipient(oResource.Alias)
        olRecipient.Resolve
      End If
            
      If Not olRecipient.Resolved Then
        ' The user name given is not unique - could not resolve
        gsResults = gsResults & " - (skipped) NOT RECOGNIZED! " & vbCrLf
        bSkip = True
      End If
    End If
      
    If (bAll Or oResource.Selected) And Not bSkip Then
      
      DoEvents
      If frmProgressBar.Cancel Then
        Call DisplayRebuildCancelNotice
        GoTo GetAppointMents_Exit
      End If
      
      On Error Resume Next
      Set olMAPIFolder = Nothing

      If oResource.Alias = oResource.FullName Then
        ' Alias = FullName only if Alias was never included
        ' So we should reassign alias with Address of recipient
        oResource.Alias = BreakoutAlias(olRecipient.Address)
      End If
      
      ' Add new resource item (employee name) to schedocx
      nResult = frmCalView.schAppointments.AddResource(nLBCount, _
        oResource.DisplayName, nLBCount, "", "", "", "", 28, -1, -1, "", 60, False, False)
        
      ' Display name
      frmProgressBar.lblCurrentName.Caption = olRecipient.Name
      
      ' Two differing GetFolder methods
      If olRecipient.Name = golNamespace.CurrentUser Then
        ' Get Appointment Items for Current CalView/Outlook user
        Set olMAPIFolder = golNamespace.GetDefaultFolder(olFolderCalendar)
      Else
        ' Get Appointment Items for other names
        Set olMAPIFolder = golNamespace.GetSharedDefaultFolder(olRecipient, olFolderCalendar)
      End If
      sErrorDesc = Err.Description & "(" & ConvertMapiError(Err.Number) & ")"
      
      On Error GoTo GetAppointMents_Error
      
      If Not (olMAPIFolder Is Nothing) Then
        If Not FillIndividualCalItems(olMAPIFolder, sCriteria, _
                olRecipient, nTBIndex, nLBCount) Then
          ' Must have cancelled
          GoTo GetAppointMents_Exit
        End If
      Else
        ' Need to alert user of failure to obtain any calendar data
        gsResults = gsResults & " - " & sErrorDesc & vbCrLf
      End If
      
      ' Update progress bar if required
      If bProgressBar Then
        frmProgressBar.pbrNames.Value = nPBNameCounter
      End If
    End If
  Next nLBCount
    
GetAppointMents_Exit:
  On Error Resume Next
  
  ' Cleanup code
  Set olRecipient = Nothing
  Set olMAPIFolder = Nothing
  
  Exit Sub

GetAppointMents_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With

  If gbDesignTime Then
    Stop
    Resume
  Else
    GoTo GetAppointMents_Exit
    Resume 'Does Nothing - For Debugging Purposes
  End If
    
End Sub

Private Function FillIndividualCalItems(olMAPIFolder As Outlook.MAPIFolder, sCriteria As String, olRecipient As Outlook.Recipient, ByRef nTBIndex As Integer, nLBCount As Integer) As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose
'Inputs         olMAPIFolder
'               sCriteria
'               olRecipient
'               ByRef nTBIndex
'               nLBCount
'Assumes
'Returns        T = Keep going, F = Cancelled
'Effects
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "FillIndividualCalItems"    ' Error logging

Dim olCalendarItemsAll As Outlook.Items
Dim olCalendarItemsRestricted As Outlook.Items
Dim objAppt As AppointmentItem
Dim oItem As Object
  
Dim nResult As Integer
Dim nArraycount As Integer
Dim nBarColor As Integer
Dim lDuration As Long
Dim lSensitivity As Long

Dim bAcceptAppt As Boolean
Dim sCategories As String
Dim sStartDate As String
Dim sEndDate As String
Dim sLocation As String
Dim sSubject As String

Dim sEntryID As String
  
  On Error GoTo FillIndividualCalItems_Error
    
  ' By default, keep going
  FillIndividualCalItems = True
  
  ' Set reference to Items Collection
  Set olCalendarItemsAll = olMAPIFolder.Items
  
  ' Sort by start date so that Reoccurring Appointments are created properly
  olCalendarItemsAll.Sort "[Start]", False
  
  ' Include recurring appointments when restricting
  olCalendarItemsAll.IncludeRecurrences = True
  
  ' Request appointments only for date specified in sCriteria
  ' (ie) [MessageClass]="IPM.Appointment" AND [Start]>="3/7/98 0:00 AM" AND [Start]<="3/14/98 11:59 PM"
  ' Note that criteria string passed to Restrict must include quotes as shown in sample
  Set olCalendarItemsRestricted = olCalendarItemsAll.Restrict(sCriteria)
  
  ' Resort as Reoccuring appointments might have been created
  olCalendarItemsRestricted.Sort "[Start]", False
                              
  ' If we found a criteria match

  For Each oItem In olCalendarItemsRestricted
  
    On Error GoTo FillIndividualCalItems_Recover1
    
    DoEvents
    If frmProgressBar.Cancel Then
      Call DisplayRebuildCancelNotice
      FillIndividualCalItems = False
      GoTo FillIndividualCalItems_Exit
    End If
    
    ' Referencing .duration/.catagories/.sensitivity on calendars with data imported from Schedule+
    ' has occasionally caused an Outlook error
    ' (ie) Error 17389 - Method 'Duration' of object 'Appointmentitem' failed
    '    or Error 440 - Automation error
    '
    sStartDate = oItem.Start
    sEndDate = oItem.End
    sLocation = oItem.Location
    sSubject = oItem.Subject

    On Error Resume Next
    lDuration = oItem.Duration
    If Err.Number <> 0 Then
      lDuration = DateDiff("n", sStartDate, sEndDate)
    End If
    
    On Error GoTo FillIndividualCalItems_Error
    
    ' Assume appointment with no time allotted are reminders and not included in view
    If lDuration > 0 Then
      On Error Resume Next

      ' Set empty in case reference to object.categories or sensitivity bombs
      sCategories = ""
      sCategories = oItem.Categories

      ' Sensitivity indicates Normal/Personal/Private/Confidential status
      ' set to normal just incase of Schedule+ error
      lSensitivity = olNormal
      lSensitivity = oItem.Sensitivity
      
      On Error GoTo FillIndividualCalItems_Error

      ' If this is other than normal appointment then include timeblock
      ' but hide details of the appointment. Keep this data on user's own view
      If lSensitivity <> olNormal And olRecipient.Name <> golNamespace.CurrentUser Then
        Select Case lSensitivity
          Case olPersonal
            sSubject = "Personal"
          
          Case olPrivate
            sSubject = "Private"
          
          Case olConfidential
            sSubject = "Confidential"
          
          Case Else
            ' Do Nothing
        End Select
        
        sLocation = ""
        sCategories = ""
      End If 'nSensitivity <> olNormal And olRecipient.Name <> golNamespace.CurrentUser
                              
      ' Compare categories if requested
      If frmCalView.chkCategory Then
        bAcceptAppt = False
        For nArraycount = 0 To UBound(gasCategoriesToInclude)
          If InStr(1, sCategories, gasCategoriesToInclude(nArraycount), 1) > 0 Then
            bAcceptAppt = True
            Exit For
          End If
        Next
      Else
        bAcceptAppt = True
      End If
      
      ' If appointment is on Sat/Sun then include only if Weekend checked
      If frmCalView.chkWeekends = vbUnchecked And (Weekday(sStartDate) = vbSaturday Or Weekday(sStartDate) = vbSunday) Then
        bAcceptAppt = False
      End If
      
      If bAcceptAppt Then
        ' Put item in schedule
        ' nResult = frmCalView.schAppointments.AddTimeBlock(nTBIndex, nLBCount, sStartDate, sEndDate, "", nLBCount, -1, sSubject, sLocation, sCategories, "")
        nBarColor = GetBarColor(sLocation)
        nResult = frmCalView.schAppointments.AddTimeBlock(nTBIndex, nLBCount, sStartDate, sEndDate, "", nBarColor, -1, sSubject, sLocation, sCategories, "")
        
        ' Increment timeblock index
        nTBIndex = nTBIndex + 1
        
        ' Make sure there is timeblock room
        If nTBIndex >= frmCalView.schAppointments.TBMax Then
          If frmCalView.schAppointments.TBMax + 100 < MAXIMUM_APPOINTMENTS Then
            frmCalView.schAppointments.TBMax = nTBIndex + 100
          Else
            frmCalView.schAppointments.TBMax = MAXIMUM_APPOINTMENTS
          End If
          ' If index counter is still higher than max timeblocks then we
          ' Have no more array room to play with. time to leave
          If nTBIndex > frmCalView.schAppointments.TBMax Then
            MsgBox App.Title & " is limited to a total of " & MAXIMUM_APPOINTMENTS & " appointments", vbCritical
            Exit For
          End If
        End If ' nTBIndex >= frmCalView.schAppointments.TBMax
      
      End If ' bAcceptAppt
    End If
    
    DoEvents
    If frmProgressBar.Cancel Then
      Call DisplayRebuildCancelNotice
      FillIndividualCalItems = False
      GoTo FillIndividualCalItems_Exit
    End If
  
  Next ' oItem
  
  DoEvents
  If frmProgressBar.Cancel Then
    Call DisplayRebuildCancelNotice
    FillIndividualCalItems = False
    GoTo FillIndividualCalItems_Exit
  End If
  
  ' Update listbox to show this one is okay
  gsResults = gsResults & " - Ok" & vbCrLf

FillIndividualCalItems_Exit:
  Exit Function

FillIndividualCalItems_Recover1:
  sEntryID = oItem.EntryID
  Resume Next
Exit Function

FillIndividualCalItems_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With

  If gbDesignTime Then
    Stop
    Resume
  Else
    Resume FillIndividualCalItems_Exit
  End If

End Function

Private Function GetBarColor(sLocation As String) As Integer
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Purpose        Get which bar to access when creating a task (each bar is a different color)
'Inputs         sLocation       - Used if coloring tasks by location
'Assumes
'Returns        The ordinal value of the bar
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Dim nColor As Integer
Dim nDelim As Integer
Dim sLocationOnly As String

  On Error Resume Next
  
  nDelim = InStr(1, sLocation, "-", vbTextCompare)
  If nDelim > 0 Then
    sLocationOnly = Trim(Left(sLocation, nDelim - 1))
  Else
    sLocationOnly = sLocation
  End If
  If sLocationOnly = "" Then
    nColor = mcolLocationColors("Default")
  Else
    nColor = mcolLocationColors(sLocationOnly)
    If Err.Number > 0 Then
      Err.Number = 0
      nColor = mcolLocationColors("Default")
    End If
  End If
  If Err.Number > 0 Then
    nColor = 0
  End If
  Err.Number = 0

  GetBarColor = nColor

End Function

Public Sub AddNamesToListBox(oResources As clsResources, sKeyName As String)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Purpose        Takes names from submitted string and adds to submitted listbox control
'Inputs         lisNames  Listbox to fill
'               sKeyName  Names delimited with ";" character
'Assumes
'Returns
'Effects
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "AddNamesToListBox"    ' Error logging

Dim oResource As clsResource
Dim sNames As String
Dim sIndivName As String
Dim lRc As Long
Dim nAlias As Integer
Dim nName As Integer
    
  On Error GoTo AddNamesToListBox_Error

  ' Arbitrary max size of 8k
  sNames = Space(16000)
  
  lRc = GetPrivateProfileString("NameLists", "~" & sKeyName, "", sNames, 16000, "CalView_" & gsWindowsUserName & ".ini")
  
  ' Clear listbox
  oResources.Clear
  
  ' Add names to listbox - each name is separated by a ";"
  Do While True
    nName = InStr(sNames, ";")
    If nName > 0 Then
      sIndivName = Left(sNames, nName - 1)
      ' Extract the alias (userid)
      nAlias = InStr(sIndivName, "*")
      If nAlias > 0 Then
        Set oResource = oResources.Add(Mid(sIndivName, nAlias + 1))
        oResource.Alias = Left(sIndivName, nAlias - 1)
      Else
        Set oResource = oResources.Add(sIndivName)
        ' If no alias given, use the full name as alias
        oResource.Alias = sIndivName
      End If
      sNames = Mid(sNames, nName + 1)
    Else
      Exit Do
    End If
  Loop
  
  ' Save name of current list
  gsCurrentNameList = sKeyName
  
  ' Put name of group on form
  frmCalView.lblCurrentList.Caption = sKeyName

AddNamesToListBox_Exit:
  On Error Resume Next
  Exit Sub

AddNamesToListBox_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With
      
  If gbDesignTime Then
    Stop
    Resume
  Else
    GoTo AddNamesToListBox_Exit
    Resume 'Does Nothing - For Debugging Purposes
  End If
    
End Sub

Public Function sGetWindowsUserName() As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Purpose        Gets Windows Name current user is logged on as
'Returns        String value of name obtained from API
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sGetWindowsUserName"    ' Error logging
Dim sUserFullName As String
Dim lSize As Long
Dim lRetVal As Long

  On Error GoTo sGetWindowsUserName_Error

  lSize = 200
  sUserFullName = Space(199)
  
  lRetVal = GetUserName(sUserFullName, lSize)
  
  If lRetVal > 0 Then
    ' Strip off NULL character at end of string
    sGetWindowsUserName = Left(sUserFullName, Len(Trim(sUserFullName)) - 1)
  End If

sGetWindowsUserName_Exit:
  On Error Resume Next
  Exit Function

sGetWindowsUserName_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With
      
  If gbDesignTime Then
    Stop
    Resume
  Else
    GoTo sGetWindowsUserName_Exit
    Resume ' Does Nothing - For Debugging Purposes
  End If

End Function

Public Sub RebuildScheduleData(sStartDate As String, sEndDate As String, bAll As Boolean)

Dim nCategoryCount As Integer
Dim nArraycount As Integer
Dim sCriteria As String
    
  On Error GoTo RebuildScheduleErr
  
  frmCalView.mnLastResource = -1
  frmCalView.mnStartPoint = -1
    
  ' Make sure we have selected names
  If frmCalView.mResources.Count > 0 Then
  
    ' Check for category selections if required
    If frmCalView.chkCategory Then
      If frmCalView.lisCategory.SelCount = 0 Then
        MsgBox "You have not selected any categories to look for", vbCritical
        Exit Sub
      Else
        '// build an array of only those categories required
        nArraycount = 0
        ReDim gasCategoriesToInclude(frmCalView.lisCategory.SelCount - 1)
        For nCategoryCount = 0 To frmCalView.lisCategory.ListCount - 1
          If frmCalView.lisCategory.Selected(nCategoryCount) = True Then
            gasCategoriesToInclude(nArraycount) = frmCalView.lisCategory.List(nCategoryCount)
            nArraycount = nArraycount + 1
          End If
        Next
      End If
    End If
      
    '// build date part of restrict filter string
    '// We want only appointment items
    sCriteria = "[MessageClass]=""IPM.Appointment"" AND [Start]>="""
    sCriteria = sCriteria & sStartDate & " 0:00 AM"""
    sCriteria = sCriteria & " AND [Start]<=""" & sEndDate & " 11:59 PM"""
        
    Screen.MousePointer = vbHourglass
        
    frmCalView.schAppointments.Redraw = False
            
    '// true rebuilds all schedules
    If bAll Then
      ' Kill all ocx resources (names)
      frmCalView.schAppointments.ResClear
      ' Kill all timeblocks (appointments)
      frmCalView.schAppointments.TBClear
    End If
    
    ' Put the progress bar up to keep people occupied
    frmCalView.vstabCalView.CurrTab = 1
    
    frmCalView.Enabled = False
    With frmProgressBar
      Call .Display(True)
      .pbrNames.Visible = True
      .ZOrder vbBringToFront
      .pbrNames.Max = IIf(bAll, frmCalView.mResources.Count, frmCalView.mResources.SelCount)
      .lblProgressMessage.Caption = "Loading Schedules..."
      .pbrNames.Value = frmProgressBar.pbrNames.Min
    End With
    
    DoEvents
    
    ' Go get actual outlook calendar data
    GetAppointments frmCalView.mResources, sCriteria, bAll, True
  Else
    ' Kill all resources (names)
    frmCalView.schAppointments.ResClear
    ' Kill all timeblocks (appointments)
    frmCalView.schAppointments.TBClear
      
  End If
  
  ' Flag for is schedule up to date
  gbScheduleUpdated = True
  
  With frmCalView
    ' Set TimeScale begin date
    .schAppointments.TSDateTimeBeg(0) = sStartDate & " " & .msStartTime
    ' Set TimeScale End Data
    .schAppointments.TSDateTimeEnd(0) = sEndDate & " " & .msEndTime
    ' Start view at 8am on first day selected - 8am is arbitrary
    .schAppointments.TSDateTimeView(0) = sStartDate & " " & .msStartTime

    .schAppointments.Redraw = True
  End With
  
  ' Get rid of progressbar form
  frmCalView.Enabled = True
  frmProgressBar.Hide
        
RebuildScheduleExit:
  On Error Resume Next
  Screen.MousePointer = vbDefault
  frmCalView.hsbSchedule.Tag = DateDiff("d", sStartDate, sEndDate) + 1
  Exit Sub

RebuildScheduleErr:
  Resume RebuildScheduleExit

End Sub

Public Sub GetOptions()
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Purpose        Sets up Select tab values and loads default name list
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "GetOptions"    ' Error logging
    
Dim nRc As Long
Dim sOption As String
Dim nOption As Integer
Dim sStartTime As String
Dim sEndTime As String

  On Error GoTo GetOptions_Error

  gsDefaultNameList = Space(22)
  nRc = GetPrivateProfileString("Options", "DefList", "", gsDefaultNameList, 22, "CalView_" & gsWindowsUserName & ".ini")
  ' Return value will have chr(0) appended - remove it
  gsDefaultNameList = Left(gsDefaultNameList, Len(Trim(gsDefaultNameList)) - 1)
  
  If gsDefaultNameList = "" Then
    frmCalView.lblCurrentList.Caption = "NONE"
  Else
    ' Add names from default name list to nameslistbox
    AddNamesToListBox frmCalView.mResources, Mid(gsDefaultNameList, 2)
  End If
  
  ' Do we include weekends when finding tasks?
  sOption = Space(5)
  nRc = GetPrivateProfileString("Options", "Weekends", "", sOption, 6, "CalView_" & gsWindowsUserName & ".ini")
  If Left(sOption, 4) = "True" Then
    frmCalView.chkWeekends.Value = vbChecked
  Else
    frmCalView.chkWeekends.Value = vbUnchecked
  End If
    
  ' Was this window maximized last time we used it?
  sOption = Space(5)
  nRc = GetPrivateProfileString("Options", "Maximized", "", sOption, 6, "CalView_" & gsWindowsUserName & ".ini")
  If Left(sOption, 4) = "True" Then
    CenterForm frmCalView
    frmCalView.WindowState = vbMaximized
  Else
    frmCalView.WindowState = vbNormal
  End If
  
  ' What is our default view?
  sOption = Space(20)
  nRc = GetPrivateProfileString("Options", "DefView", "", sOption, 21, "CalView_" & gsWindowsUserName & ".ini")
  If Left(sOption, 5) = "Daily" Then
    frmCalView.cboDefaultView.ListIndex = 0
  ElseIf Left(sOption, 7) = "Monthly" Then
    frmCalView.cboDefaultView.ListIndex = 2
  Else
    ' All other cases should be weekly
    frmCalView.cboDefaultView.ListIndex = 1
  End If
  
  ' Daily view start/end time
  sOption = Space(20)
  nRc = GetPrivateProfileString("Options", "TimeStart", "", sOption, 21, "CalView_" & gsWindowsUserName & ".ini")
  sStartTime = Left(sOption, 7)
  sOption = Space(20)
  nRc = GetPrivateProfileString("Options", "TimeEnd", "", sOption, 21, "CalView_" & gsWindowsUserName & ".ini")
  sEndTime = Left(sOption, 7)
  
  On Error Resume Next
  frmCalView.cboDailyTimeStart.Text = sStartTime
  If Err.Number > 0 Then
    Err.Number = 0
    frmCalView.cboDailyTimeStart.Text = frmCalView.msStartTime
  Else
    frmCalView.msStartTime = sStartTime
  End If
  
  frmCalView.cboDailyTimeEnd.Text = sEndTime
  If Err.Number > 0 Then
    Err.Number = 0
    frmCalView.cboDailyTimeEnd.Text = frmCalView.msEndTime
  Else
    frmCalView.msEndTime = sEndTime
  End If
  
  On Error GoTo GetOptions_Error
  
  ' Default start will be today
  frmCalView.txtStartDate = Date
  
  ' Default end will be 7 days from today
  frmCalView.txtEndDate = Date + 7
  
  ' Categories
  sOption = vGetRegistryValue(CATEGORY_SEARCH_PATH, "")

  If sOption = "" Then
    sOption = sSetDefaultCategories
  End If
  ' Add categories to listbox - each is separated by a ";"
  Do While True
    nOption = InStr(sOption, ";")
    If nOption > 0 Then
      frmCalView.lisCategory.AddItem (Left(sOption, nOption - 1))
      sOption = Right$(sOption, Len(sOption) - nOption)
    Else
      Exit Do
    End If
  Loop

GetOptions_Exit:
  On Error Resume Next
  Exit Sub

GetOptions_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With
      
  If gbDesignTime Then
    Stop
    Resume
  Else
    Resume GetOptions_Exit
  End If

End Sub

Public Function sSetDefaultCategories() As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Creates and saves to registry the standard Outlook Categories
'Returns        String of Categories
'Effects        CATEGORY_SEARCH_PATH Registry Key is updated
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
    ' Declarations:
    Const sPROCEDURE_NAME       As String = "sSetDefaultCategories"    ' Error logging
    Dim nRc As Long
    Dim sErrMsg As String

    ' Errors
    On Error GoTo sSetDefaultCategories_Error
    
    sSetDefaultCategories = "Business;Competition;Favorites;Gifts;Goals/Objectives;Holiday;Holiday Cards;Hot Contacts;"
    sSetDefaultCategories = sSetDefaultCategories & "Ideas;International;Key Customer;Miscellanous;Personal;Phone Calls;"
    sSetDefaultCategories = sSetDefaultCategories & "Status;Strategies;Suppliers;Time & Expenses;VIP;Waiting;"

    nRc = CreateNewRegistryKey(CATEGORY_SEARCH_PATH, HKEY_CURRENT_USER)
    
    If nRc = ERROR_NONE Then
        nRc = SetRegistryValue(CATEGORY_SEARCH_PATH, "", sSetDefaultCategories, REG_SZ)
        If nRc <> ERROR_NONE Then
            sErrMsg = "Unable to save default Category information to the Windows Registry key: " & CATEGORY_SEARCH_PATH
        End If
    Else
        sErrMsg = "Unable to Create the Windows Registry key: " & CATEGORY_SEARCH_PATH
    End If
    
    If sErrMsg <> "" Then
        MsgBox sErrMsg, vbCritical, App.Title
    End If

sSetDefaultCategories_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sSetDefaultCategories_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sSetDefaultCategories_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
        
End Function

Public Sub PrintAppointments()
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Prints All appointments currently in SchOCX control
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "PrintAppointments"    ' Error logging

' Errors
On Error GoTo PrintAppointments_Error

    Dim nPageNo As Integer
    Dim nTBCount As Integer
    Dim nRowNo As Integer
    
    Screen.MousePointer = vbHourglass
    
    Printer.Orientation = vbPRORLandscape
    
    nPageNo = 1
    nRowNo = 6
    PrintApptTop nPageNo
    
    For nTBCount = 0 To frmCalView.schAppointments.TBMax - 1
        
        '// tbbar will return -1 if not initialized (ie) end of data
        If frmCalView.schAppointments.TBBar(nTBCount) = -1 Then
            Exit For
        End If
                
        If nRowNo > 45 Then
            nRowNo = 6
            nPageNo = nPageNo + 1
            PrintApptTop nPageNo
        End If
        
        'Sandy Chin replace the following statements at 9/9/98
        '// limit name to 1st 25 characters, subject & location to 1st 40 characters
        'PrintRowCol nRowNo, 1, frmCalView.schAppointments.TBBeg(nTBCount)
        'PrintRowCol nRowNo, 15, ConvertDurationToMinutes(frmCalView.schAppointments.TBDur(nTBCount)) & " min"
        'PrintRowCol nRowNo, 25, Left(frmCalView.schAppointments.ResName(frmCalView.schAppointments.TBResource(nTBCount)), 25)
        'PrintRowCol nRowNo, 46, Left(frmCalView.schAppointments.TBUser1(nTBCount), 30)
        'PrintRowCol nRowNo, 71, Left(frmCalView.schAppointments.TBUser2(nTBCount), 30)
        
        PrintRowCol nRowNo, 1, frmCalView.schAppointments.TBBeg(nTBCount)
        PrintRowCol nRowNo, 22, frmCalView.schAppointments.TBEnd(nTBCount)
        PrintRowCol nRowNo, 42, Left(frmCalView.schAppointments.ResName(frmCalView.schAppointments.TBResource(nTBCount)), 25)
        PrintRowCol nRowNo, 65, Left(frmCalView.schAppointments.TBUser1(nTBCount), 40)
        PrintRowCol nRowNo, 100, Left(frmCalView.schAppointments.TBUser2(nTBCount), 30)
        
        'Sandy Chin comment out the following statement at 9/9/98
        'PrintRowCol nRowNo, 97, Left(frmCalView.schAppointments.TBUser3(nTBCount), 25)
        
        nRowNo = nRowNo + 1

    Next
    
    Printer.EndDoc
    
    Screen.MousePointer = vbDefault

PrintAppointments_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
PrintAppointments_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo PrintAppointments_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Sub

Private Function ConvertDurationToMinutes(sTBDuration As String) As Long
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Converts SchedOCX Duration value from string to long of total minutes
'Inputs         sTBDuration     - Duration property
'Assumes        Standard Schedocx duration format is used - "##days ##hours ##minutes"
'Returns        Total minutes in this appointment
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
    Const sPROCEDURE_NAME       As String = "ConvertDurationToMinutes"    ' Error logging
    Dim nDays As Long
    Dim nHours As Long
    Dim nMinutes As Long

' Errors
On Error GoTo ConvertDurationToMinutes_Error

    ConvertDurationToMinutes = 0
    
    nDays = Val(sTBDuration)
    nHours = Val(Mid(sTBDuration, 8, 2))
    nMinutes = Val(Mid(sTBDuration, 16, 2))
    ConvertDurationToMinutes = (nDays * 1440) + (nHours * 60) + nMinutes
    
ConvertDurationToMinutes_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
ConvertDurationToMinutes_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo ConvertDurationToMinutes_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Function
Private Sub PrintApptTop(nPageNo As Integer)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        What the routine does (not how).
'Inputs         Each non-obvious parameter on a separate line with in-line comments
'Assumes        List of each non-obvious external variable, control, open file and so on.
'Returns        Explanation of value returned for functions.
'Effects        List of each effected external variable, control, file, and
'           so on and the affect it has (only if this is not obvious)
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
    ' Declarations:
    Const sPROCEDURE_NAME       As String = "PrintApptTop"    ' Error logging

    ' Errors
    On Error GoTo PrintApptTop_Error

    Printer.NewPage
    Printer.FontName = "Courier New"
    Printer.FontSize = 24
    PrintCenteredText 0, "Appointment Schedule"
    
    Printer.FontSize = 8
    Printer.FontBold = True
    PrintRowCol 2, 1, Format(Now, "dddd mmm dd, yyyy hh:mm")
    PrintRowCol 2, 105, "Page: " & nPageNo
    
    PrintCenteredText 3, Format(frmCalView.txtStartDate, "dddd mm/dd/yyyy") & " thru " & Format(frmCalView.txtEndDate, "dddd mm/dd/yyyy")
    
    Printer.FontUnderline = True
    'Sandy Chin replace the following statements at 9/9/98
    'PrintRowCol 5, 1, "Time" & Space(12)
    'PrintRowCol 5, 15, "Duration"
    'PrintRowCol 5, 25, "Name" & Space(21)
    'PrintRowCol 5, 46, "Subject" & Space(23)
    'PrintRowCol 5, 71, "Location" & Space(22)
    'PrintRowCol 5, 97, "Category" & Space(18)
    
    PrintRowCol 5, 1, "Start Date" & Space(10)
    PrintRowCol 5, 22, "End Date" & Space(12)
    PrintRowCol 5, 42, "Name" & Space(21)
    PrintRowCol 5, 65, "Subject" & Space(33)
    PrintRowCol 5, 100, "Location" & Space(22)
    
    Printer.FontBold = False
    Printer.Print
    Printer.FontUnderline = False

PrintApptTop_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
PrintApptTop_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo PrintApptTop_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Sub
    
Public Sub PrintCenteredText(nRowNo, sStuffToPrint)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Calculates start point for centered text then prints it
'Inputs         nRowNo          What line to print on
'               sStuffToPrint   Stuff to print!
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
    ' Declarations:
    Const sPROCEDURE_NAME       As String = "PrintCenteredText"    ' Error logging
    Dim nHalfWidth As Single
    Dim nCurrentMode As Integer
    

    ' Errors
    On Error GoTo PrintCenteredText_Error

    If Not IsEmpty(sStuffToPrint) Then
    
        nCurrentMode = Printer.ScaleMode
        Printer.ScaleMode = 4
    
        Printer.CurrentY = nRowNo

        nHalfWidth = Printer.TextWidth(sStuffToPrint) / 2
        Printer.CurrentX = Printer.ScaleWidth / 2 - nHalfWidth

        Printer.Print sStuffToPrint
    
        Printer.ScaleMode = nCurrentMode
    End If


PrintCenteredText_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
PrintCenteredText_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo PrintCenteredText_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Sub
Public Sub PrintRowCol(nRow As Integer, nCol As Single, sStuffToPrint As String)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Prints at designed row/col start point
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
    ' Declarations:
    Const sPROCEDURE_NAME       As String = "PrintRowCol"    ' Error logging
    Dim nCurrentMode As Integer
    
    ' Errors
    On Error GoTo PrintRowCol_Error

    If Not IsEmpty(sStuffToPrint) Then
    
        nCurrentMode = Printer.ScaleMode
        Printer.ScaleMode = 4
    
        Printer.CurrentY = nRow
        Printer.CurrentX = nCol
        Printer.Print sStuffToPrint
    
        Printer.ScaleMode = nCurrentMode
    End If

PrintRowCol_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
PrintRowCol_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo PrintRowCol_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Sub

Public Function vGetRegistryValue(sKeyName As String, sValuename As String) As Variant
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        What the routine does (not how).
'Inputs         Each non-obvious parameter on a separate line with in-line comments
'Assumes        List of each non-obvious external variable, control, open file and so on.
'Returns        Explanation of value returned for functions.
'Effects        List of each effected external variable, control, file, and
'           so on and the affect it has (only if this is not obvious)
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
    ' Declarations:
    Const sPROCEDURE_NAME       As String = "vGetRegistryValue"    ' Error logging

    ' Errors
    On Error GoTo vGetRegistryValue_Error

    Dim lRetVal As Long
    Dim lhKey As Long
    Dim vValue As Variant
    
    '// set empty string as error return value
    vValue = ""
    
    '// attempt to open the registry path
    lRetVal = RegOpenKeyEx(HKEY_CURRENT_USER, sKeyName, 0, KEY_ALL_ACCESS, lhKey)
    
    '// if the path/key opened then get the value
    If lRetVal = ERROR_NONE Then
        lRetVal = QueryValueEx(lhKey, sValuename, vValue)
        
        RegCloseKey (lhKey)
    End If
    
    vGetRegistryValue = vValue


vGetRegistryValue_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
vGetRegistryValue_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo vGetRegistryValue_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
    
End Function

Private Function QueryValueEx(ByVal lhKey As Long, ByVal szValueName As String, vValue As Variant) As Long

Dim cch As Long
Dim lRc As Long
Dim ltype As Long
Dim lValue As Long
Dim sValue As String
    
    On Error GoTo QueryValueExError
    
    lRc = RegQueryValueExNULL(lhKey, szValueName, 0&, ltype, 0&, cch)
    
    If lRc <> ERROR_NONE Then Error 5
    
    Select Case ltype
        '// Strings
        Case REG_SZ
            sValue = String(cch, 0)
            
            lRc = RegQueryValueExSTRING(lhKey, szValueName, 0&, ltype, sValue, cch)
            If lRc = ERROR_NONE Then
                vValue = Left$(sValue, cch)
            Else
                vValue = Empty
            End If
        
        '// for DWORDS
        Case REG_DWORD
            lRc = RegQueryValueExLONG(lhKey, szValueName, 0&, ltype, lValue, cch)
            If lRc = ERROR_NONE Then vValue = ltype
        Case Else
            lRc = -1
    End Select
        
QueryValueExExit:
    QueryValueEx = lRc
    Exit Function
    
QueryValueExError:
    Resume QueryValueExExit
            
End Function
Public Function SetRegistryValue(sKeyName As String, sValuename As String, vValueSetting As Variant, lValueType As Long) As Long
    Dim hKey As Long

'// sKeyname = Path
'// sValueName = Name of setting
'// vValueSetting = New value to set
'// lValueType = Data type of setting (string or long)
'//    Example - lngRetVal = SetRegistryValue("Software\Microsoft\Visual Basic\5.0", "AlignToGrid", "1", REG_SZ)
    
    SetRegistryValue = RegOpenKeyEx(HKEY_CURRENT_USER, sKeyName, 0, KEY_ALL_ACCESS, hKey)
    
    If SetRegistryValue = ERROR_NONE Then
        SetRegistryValue = SetValueEx(hKey, sValuename, lValueType, vValueSetting)
        RegCloseKey (hKey)
    End If
    
End Function

Private Function SetValueEx(ByVal hKey As Long, sValuename As String, ltype As Long, vValue As Variant) As Long

Dim lValue As Long
Dim sValue As String
    
    Select Case ltype
        Case REG_SZ
            sValue = vValue & Chr$(0)
            SetValueEx = RegSetValueExString(hKey, sValuename, 0&, ltype, sValue, Len(sValue))
            
        Case REG_DWORD
            lValue = vValue
            SetValueEx = RegSetValueExLong(hKey, sValuename, 0&, ltype, lValue, 4)
        End Select
        
End Function

Public Function CreateNewRegistryKey(strNewKeyName As String, lngPredefinedKey As Long)
    Dim hNewKey As Long
    Dim lngRetVal As Long
    
    lngRetVal = RegCreateKeyEx(lngPredefinedKey, strNewKeyName, 0&, vbNullString, _
        REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, 0&, hNewKey, lngRetVal)
    
    RegCloseKey (hNewKey)
    
    CreateNewRegistryKey = lngRetVal
    
End Function

Public Sub SetHSBValues()

Dim lRange As Long

  On Error Resume Next

  lRange = frmCalView.hsbSchedule.Tag - 1
  If Not frmCalView.cmdViewDaily.Enabled Then
    frmCalView.hsbSchedule.Max = lRange
  ElseIf Not frmCalView.cmdViewWeekly.Enabled Then
    frmCalView.hsbSchedule.Max = lRange \ 7
  Else
    frmCalView.hsbSchedule.Max = lRange \ 31
  End If

  frmCalView.hsbSchedule.Value = frmCalView.hsbSchedule.Min

End Sub

Public Sub SetViewToDaily()
    
    With frmCalView
    
      With .schAppointments
        .Redraw = False
        .DisplayMode = DisplayMode_ResView
    
        .TSMinorFontSizePercent(0) = 75
        .TSMinor(0) = 5
        .TSMinorCount(0) = 1

        .TSMinorFormat(0) = 1
        .TSNumMinorTicks(0) = DateDiff("h", frmCalView.msStartTime, frmCalView.msEndTime)
    
        .TSMinorGrid(0) = False
    
        .TSMajor(0) = 4
        .TSMajorCount(0) = 0
    
        .TSMajorFormat(0) = 14
    
        .TSRulerLarge(0) = .TSMinorCount(0) - 1
        ' If there are only a few tick marks...sub-divide (twice as many)
        If .TSRulerLarge(0) < 4 Then
          .TSRulerLarge(0) = .TSRulerLarge(0) * 2 + 1
        End If

        .TSMajorGrid(0) = True
      End With
    
      .cmdViewDaily.Enabled = False
      .cmdViewMonthly.Enabled = True
      .cmdViewWeekly.Enabled = True
    
      .mnuContextMenuViewDaily = False
      .mnuContextMenuViewWeekly = True
      .mnuContextMenuViewMonthly = True
    
      .schAppointments.Redraw = True

    End With
    
    Call SetHSBValues

End Sub
Public Sub SetViewToWeekly()
    
    frmCalView.schAppointments.Redraw = False
    
    frmCalView.schAppointments.DisplayMode = DisplayMode_ResView
    
    frmCalView.schAppointments.TSMinorFontSizePercent(0) = 50
    
    frmCalView.schAppointments.TSMinor(0) = 4           'show days
    frmCalView.schAppointments.TSMinorCount(0) = 1      '1 cell per day
    frmCalView.schAppointments.TSMinorFormat(0) = 3     'Fri Sep 5
    frmCalView.schAppointments.TSNumMinorTicks(0) = 7   'Show 7 days
    frmCalView.schAppointments.TSMinorGrid(0) = True
    
    frmCalView.schAppointments.TSMajor(0) = 3           'ok
    frmCalView.schAppointments.TSMajorCount(0) = 0      'ok
    frmCalView.schAppointments.TSMajorFormat(0) = 2     'Ok
    frmCalView.schAppointments.TSRulerLarge(0) = 0      'ok
    frmCalView.schAppointments.TSMajorGrid(0) = True
        
    frmCalView.cmdViewDaily.Enabled = True
    frmCalView.cmdViewMonthly.Enabled = True
    frmCalView.cmdViewWeekly.Enabled = False
    
    frmCalView.mnuContextMenuViewDaily = True
    frmCalView.mnuContextMenuViewWeekly = False
    frmCalView.mnuContextMenuViewMonthly = True
    
    frmCalView.schAppointments.Redraw = True
    
    Call SetHSBValues

End Sub
Public Sub SetViewToMonthly()
    
    frmCalView.schAppointments.Redraw = False
    
    frmCalView.schAppointments.DisplayMode = DisplayMode_ResView
    
    frmCalView.schAppointments.TSMinorFontSizePercent(0) = 40
    
    frmCalView.schAppointments.TSMinor(0) = 4           'Display days
    frmCalView.schAppointments.TSMinorCount(0) = 1      '1 cell per day
    frmCalView.schAppointments.TSMinorFormat(0) = 0     'Show day of month (1st,2nd etc..)
    frmCalView.schAppointments.TSNumMinorTicks(0) = 31  'show 31 days
    frmCalView.schAppointments.TSMinorGrid(0) = True
    
    frmCalView.schAppointments.TSMajor(0) = 2           'ok
    frmCalView.schAppointments.TSMajorCount(0) = 0      'ok
    frmCalView.schAppointments.TSMajorFormat(0) = 1     'Ok
    frmCalView.schAppointments.TSRulerLarge(0) = 0      'ok
    frmCalView.schAppointments.TSMajorGrid(0) = True
        
    frmCalView.cmdViewDaily.Enabled = True
    frmCalView.cmdViewWeekly.Enabled = True
    frmCalView.cmdViewMonthly.Enabled = False
    
    frmCalView.mnuContextMenuViewDaily = True
    frmCalView.mnuContextMenuViewWeekly = True
    frmCalView.mnuContextMenuViewMonthly = False
    
    frmCalView.schAppointments.Redraw = True

    Call SetHSBValues

End Sub


Public Function eGetOSVersion() As eOSPlatform
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Obtains Version of Windows
'Returns        enum Value reflecting OS
'                   1   = Windows 95
'                   2   = Windows NT
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "eGetOSVersion"    ' Error logging
    
    ' Errors
    On Error GoTo eGetOSVersion_Error

    Dim lRetVal As Long
    Dim stOSInfo As OSVERSIONINFO
    
    stOSInfo.dwOSVersionInfoSize = 148
    stOSInfo.szCSDVersion = Space$(128)
    
    lRetVal = GetVersionEx(stOSInfo)
    
    eGetOSVersion = stOSInfo.dwPlatformId

eGetOSVersion_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
eGetOSVersion_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo eGetOSVersion_Exit
        Resume 'Does Nothing - For Debugging Purposes
        End If
    
End Function

Public Function sGetDefaultProfile() As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Obtains Version specific Windows Messaging default profile name
'Inputs         None
'Assumes
'Returns        String containing default profile name or empty string on error
'Effects
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "sGetDefaultProfile"    ' Error logging

Dim objRegistryAgent As clsRegistryAgent
Dim eOSVersion As eOSPlatform
Dim sRegistryKeyValue As String
    
    On Error GoTo sGetDefaultProfile_Error

    Set objRegistryAgent = New clsRegistryAgent
    
    eOSVersion = eGetOSVersion
    
    With objRegistryAgent
      .hKey = HKEY_CURRENT_USER
      Select Case eOSVersion
        Case osWin95
            'Win95 Registry setting
            .RegistryKey = "Software\Microsoft\Windows Messaging Subsystem\Profiles"
        
        Case osWinNT
            'WinNt Registry setting
            .RegistryKey = "Software\Microsoft\Windows NT\CurrentVersion\" & _
            "Windows Messaging Subsystem\Profiles"
        
        Case Else
            'big error
            MsgBox "Unable to locate Windows Messaging Default Profile", vbCritical + vbOKOnly, App.Title
            GoTo sGetDefaultProfile_Exit
      End Select
      
      .SubKey = "DefaultProfile"
      .GetStringValue
      sRegistryKeyValue = .Value
        
    End With

    sGetDefaultProfile = sRegistryKeyValue
    
sGetDefaultProfile_Exit:
  On Error Resume Next
  'Put cleanup code here
  Exit Function

sGetDefaultProfile_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With
      
  If gbDesignTime Then
    Stop
    Resume
  Else
    Resume sGetDefaultProfile_Exit
  End If
    
End Function

Public Function BreakoutAlias(sNameParam As String) As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Given a MAPI address string, extract the user id (alias)
'Assumes        sNameParam is a valid MAPI address string
'               (e.g. /o=Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic/ou=MMA_NY/cn=Recipients/cn=prudentid)
'               Also, assume "/cn=" is the last element delimited by "/" and
'               that IT holds the alias
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME As String = "BreakoutAlias"    ' Error logging

Dim nDelim As Integer
Dim sName As String
Dim sAlias As String

  On Error GoTo BreakoutAlias_Error

  sAlias = sNameParam

'---------------------------------------------------------------------------
' This code was removed...apparently, only using the last CN doesn't always
' work...now we use the entire .Address property (return what was passed so
' we don't have to change any code now or in the future)
'---------------------------------------------------------------------------
'  sName = sNameParam
'  ' Use only the text to the right of the last "/"
'  Do While InStr(1, sName, "/", vbTextCompare) > 0
'    sName = Mid(sName, InStr(1, sName, "/", vbTextCompare) + 1)
'  Loop
'
'  nDelim = InStr(1, sName, "cn=", vbTextCompare)
'  If nDelim > 0 Then
'    sAlias = Mid(sName, nDelim + 3)
'  Else
'    sAlias = ""
'  End If

BreakoutAlias_Exit:
  BreakoutAlias = sAlias
  Exit Function

BreakoutAlias_Error:
  With Err
    RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
  End With
      
  If gbDesignTime Then
    Stop
    Resume
  Else
    Resume BreakoutAlias_Exit
  End If

End Function

Public Sub DisplayResource(vIndex As Variant)
  ' For debug purposes only

Debug.Print frmCalView.mResources.Item(vIndex).FullName & " - " & _
    frmCalView.mResources.Item(vIndex).ListBoxIndex & " - " & _
    frmCalView.mResources.Item(vIndex).Key & " - " & _
    frmCalView.mResources.Item(vIndex).Alias

End Sub

Public Sub LoadColorCollection()
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Purpose        Loads colors for each location into a collection
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
'Declarations
Dim lLoop As Long
Dim lRc As Long
Dim iDelim As Integer
Dim sName As String
Dim lColor As Long
Dim sLocation As String

  On Error Resume Next
  
  Call ClearLocationColors
  
  For lLoop = 0 To 15
    sLocation = Space(100)
    lRc = GetPrivateProfileString("Locations", CStr(lLoop), "", sLocation, 100, "CalView_" & gsWindowsUserName & ".ini")
    iDelim = InStr(1, sLocation, "-", vbTextCompare)
    If iDelim > 0 Then
      sName = Left(sLocation, iDelim - 1)
      lColor = Val(Mid(sLocation, iDelim + 1))
      Call mcolLocationColors.Add(lColor, sName)
    End If
  Next lLoop

End Sub
