Attribute VB_Name = "basDesignError"
Option Explicit

'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Companion Code Module for frmDesign Error
'
'               ** THIS MODULE IS FULLY REUSABLE!  PLEASE REUSE IT! **
'
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
'Declarations
Private Const msMODULE_NAME      As String = "basDesignError"

Public gbDesignTime As Boolean      'Global Flag to indicate Design Time instead of Run Time
Public gbLogErrors As Boolean       'Global flag to indicate whether or not to write errors to a file.(set via command line)

Public gsErrorLogFile As String     'Log file where runtime errors are written if errorlogging is turned on' (/L on the command line)

Public Enum enumDesignErrorType   'Types of Design Time Errors -- Add more as needed
    detInformation = 0
    detVBCode = 1
    detSQLDatabase = 2
End Enum

Public Enum enumErrorCode            'Type of Generic (non-database) Error Codes -- Add more as needed
    ecNone = 1                          'Undefined Error
    ecNullString = 2                    'Empty (Null) String Not Permitted
    ecNullpointer = 3                   'Pointer Set to Nothing (Null) Not Permitted
    ecOutOfRange = 4                'Variable out of Range (bounds)
    ecNonNegative = 5               'Negative Values Not Permitted
    ecNonZero = 6                   'Zero Values Not Permitted
    ecNonNullPointer = 7           'Non-Null Pointer Not Permitte
    ecInvalid = 8                       'Generic Invalid Value
End Enum

'Global RunTime Error variables to allow RunTimeError Windows to be cached while a modal form is shown,
'and to display them once the modal form is unloaded.
Public Const gnMaxErrorWindows = 60                                             'Maximum amount of Error windows that can be cached.
Public gbModalWindowActive As Boolean                                           'Indicates whether or not a modal form is showing.
Public gnErrorWindowCount As Integer                                            'Number of Error windows currently stored.
Public gpfrmRunTimeErrorWindows(gnMaxErrorWindows) As frmRunTimeError           'Array of form pointers.

Public Sub LogError(ByVal sModuleName As String, ByVal sProcName As String, ByVal lErrCode As Long, _
                        ByVal sErrDesc As String, ByVal sErrSource As String, ByVal sHelpFile As String, _
                        ByVal lHelpContextID As Long, ByVal lLastDllError As Long)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Opens(or creates) an error log file in the application directory and appends error info
'               to it.
'Inputs         sModuleName - Source Module of the error
'               sProcName - Source Proc of the error
'               lErrCode - Number property of the VB Error object
'               sErrDesc - Description corresponding to lErrCode
'               sErrSource - Name of current Visual Basic project.
'               sHelpFile - Path and file name of the VB Help File
'               lHelpContextID - Context ID of the VB Help file corresponding to the Number property
'               lLastDLLError - Contains the sstem error code for the last call to a DLL.
'-------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "LogError"    ' Error logging

Dim lFileNum As Long, bWriteHeader As Boolean

' No error handling section in this proc in order to avoid circular reference
On Error Resume Next
        
    lFileNum = FreeFile
    
    'check whether file already exists or not
    bWriteHeader = Not bFileExists(gsErrorLogFile)
    
    'open file or create it if it doesn't exist.
    Open gsErrorLogFile For Append As #lFileNum
    
    'if this is a new file then write a header first.
    If bWriteHeader Then
        Print #lFileNum, "Date"; Tab; "Time"; Tab; "Error Code"; Tab; "Description"; Tab; "Source"; Tab; "Module"; Tab; "Procedure"; Tab; "Help File"; Tab; "Help ContextID"; Tab; "Last DLL Error"
        Print #lFileNum, "----"; Tab; "----"; Tab; "----- ----"; Tab; "-----------"; Tab; "------"; Tab; "------"; Tab; "---------"; Tab; "---- ----"; Tab; "---- ---------"; Tab; "---- --- -----"
        Print #lFileNum, 'skip a line
    End If
    
    'append error info to the file.
    Print #lFileNum, Format$(Now, "mm/dd/yyyy"); Tab; Format$(Now, "hh:nn:ss am/pm"); Tab; lErrCode; Tab; sErrDesc; _
                     Tab; sErrSource; Tab; sModuleName; Tab; sProcName; Tab; sHelpFile; Tab; lHelpContextID; Tab; lLastDllError
    Print #lFileNum, 'skip a line
    
    Close lFileNum
    
End Sub

Public Function bRunningInIDE() As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Determines whether or not the program is running in design mode
'Inputs       none
'Returns      True if in Design Mode
'-------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Dim lErrorNumber            As Long
Dim sErrorDescription       As String
Const sPROCEDURE_NAME       As String = "bRunningInIDE"    ' Error logging

' Errors
On Error GoTo bRunningInIDE_Error

    bRunningInIDE = False
    
    'Divide by zero (fails within IDE)
    Debug.Print 1 / 0
    

bRunningInIDE_Exit:
    Exit Function
    
    'Error
bRunningInIDE_Error:
    bRunningInIDE = True
    Exit Function
    
End Function


Public Function sTranslateDesignErrorType(eDesignErrorType As enumDesignErrorType) As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Translates a Design Error Type Enumeration into a String
'Inputs          eDesignErrorType Enumerator
'Returns       Text of the Design Error Type Translation
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sTranslateDesignErrorType"    ' Error logging

Dim sTranslation As String

' Errors
On Error GoTo sTranslateDesignErrorType_Error

    
    'Translate the Design Error Type
    Select Case eDesignErrorType
        Case detInformation
            sTranslation = "INFORMATIONAL"
        Case detVBCode
            sTranslation = "CODING"
        Case detSQLDatabase
            sTranslation = "SQL DATABASE"
        Case Else:
            sTranslation = "UNDEFINED"
    End Select
    

sTranslateDesignErrorType_Exit:
    On Error Resume Next
    'Put cleanup code here
    
    'Return the Translation
    sTranslateDesignErrorType = sTranslation
    
    Exit Function

' Errors:
sTranslateDesignErrorType_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    sTranslation = ""
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo sTranslateDesignErrorType_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If

End Function

Public Function sTranslateErrorCode(sErrorCode As String) As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Translates an ErrorCode Enumeration into a String
'Inputs          eErrorCode Enumerator
'Returns       Text of the ErrorCode Translation
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sTranslateErrorCode"    ' Error logging
Dim eErrorCode As enumErrorCode

Dim sTranslation As String


' Errors
On Error GoTo sTranslateErrorCode_Error

    eErrorCode = Val(sErrorCode)
    
    'Translate the ErrorCode
    Select Case eErrorCode
        Case ecNone:
            sTranslation = "No Error Details"
        Case ecNullString
            sTranslation = "Null String"
        Case ecNullpointer
            sTranslation = "Null Pointer"
        Case ecOutOfRange
            sTranslation = "Out of Range"
        Case ecNonNegative
            sTranslation = "Negatives Illegal"
        Case ecNonZero
            sTranslation = "Zero Value Illegal"
        Case ecInvalid
            sTranslation = "Invalid Value"
        Case Else:
            sTranslation = "Undefined Error"
    End Select
    

sTranslateErrorCode_Exit:
    On Error Resume Next
    'Put cleanup code here
    
    'Return the Translation
    sTranslateErrorCode = sTranslation
    
    Exit Function

' Errors:
sTranslateErrorCode_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    sTranslation = ""
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo sTranslateErrorCode_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If

End Function


Public Sub DesignTimeError(eErrorType As enumDesignErrorType, sModuleName As String, sProcName As String, sSummaryDesc As String, Optional rderrObject As RDO.rdoErrors = Nothing, Optional eErrorCode As enumErrorCode = ecNone, Optional sErrorDetail As String = "")
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Registers and displays a Design Time Error (a non-fatal, non-run time error to help developers debug their code)
'Inputs         eErrorType - Type of the Design Time Error
'                  sModuleName - Source Module of the error
'                  sProcName - Source Proc of the error
'                  sSummary - Summarizing Description of the Error
'                  rderrObject - Pointer to the RDO Error Object (use when dealing with a database error)
'                  eErrorCode - Generic Classification of the Error (an enum presents all the alternatives)
'                  sErrorDetail - Technical Detail of of the error (ex, "sName Parameter Cannot be Null")
'-------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "DesignTimeError"    ' Error logging

' Errors
On Error GoTo DesignTimeError_Error

    'Make sure the Design Time Flag is set
    If gbDesignTime Then
    
        'Load the Design Time Error Form, if not already loaded
        Load frmDesignError
    
        'Sound an Alarm
        Beep
    
        'Add the error to the form
        frmDesignError.AddError eErrorType, sModuleName, sProcName, sSummaryDesc, rderrObject, eErrorCode, sErrorDetail
    
    End If

DesignTimeError_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
DesignTimeError_Error:
    'Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo DesignTimeError_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If

End Sub
Public Sub RunTimeError(ByVal sModuleName As String, ByVal sProcName As String, ByVal lErrCode As Long, _
                        ByVal sErrDesc As String, ByVal sErrSource As String, ByVal sHelpFile As String, _
                        ByVal lHelpContextID As Long, ByVal lLastDllError As Long)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Loads Run Time Error form and sends it information about error.  Logs error to file if
'               if error logging is turned on.
'Inputs         sModuleName - Source Module of the error
'               sProcName - Source Proc of the error
'               lErrCode - Number property of the VB Error object
'               sErrDesc - Description corresponding to lErrCode
'               sErrSource - Name of current Visual Basic project.
'               sHelpFile - Path and file name of the VB Help File
'               lHelpContextID - Context ID of the VB Help file corresponding to the Number property
'               lLastDLLError - Contains the sstem error code for the last call to a DLL.
'-------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "RunTimeError"    ' Error logging

Dim pfrmRunTimeError As frmRunTimeError

' No error handling section in this proc in order to avoid circular reference
On Error Resume Next
    
    'create an instance of the RunTimeError  form
    Set pfrmRunTimeError = New frmRunTimeError
    
    'Sound an Alarm
    Beep

    'Add the error to the form
    pfrmRunTimeError.DisplayError sModuleName, sProcName, lErrCode, sErrDesc, sErrSource, _
                                 sHelpFile, lHelpContextID, lLastDllError
    
    If gbModalWindowActive = True Then
        gnErrorWindowCount = gnErrorWindowCount + 1
        Set gpfrmRunTimeErrorWindows(gnErrorWindowCount) = pfrmRunTimeError
    Else
        SetMeTopMost pfrmRunTimeError, True
        pfrmRunTimeError.Show
    End If
    
    Set pfrmRunTimeError = Nothing
    
    'If logging switch turned on then write error to Error.log
    If gbLogErrors Then
        LogError sModuleName, sProcName, lErrCode, sErrDesc, sErrSource, _
                 sHelpFile, lHelpContextID, lLastDllError
    End If
        
End Sub
Public Sub ProcessError(ByVal ModuleName As String, _
                        Optional ByVal FunctionName As String = "General Error", _
                        Optional ByVal ErrorMessage As String = "", _
                        Optional ByVal SeverityLevel As Integer = 0, _
                        Optional ByVal ErrorNumber As Long, _
                        Optional ByVal ErrorDescription As String, _
                        Optional ByVal DisplayMessage As Boolean = False, _
                        Optional ByVal LogError As Boolean = False)
'/----------------------------------------------------------
'/  This procedure will process an error.  It will optionally
'/  Display or Log the error.  It will also use the severity to
'/  determine the next action.
'/----------------------------------------------------------
'/  Parameters: Module Name         (String)
'/              Function Name       (String, Optional)
'/              Error Message       (String, Optional)
'/              Severity            (Integer, Optional, Constant)
'/              Display Error       (Boolean, Optional)
'/              Log Error           (Boolean, Optional)
'/              Clear Error         (Boolean, Optional)
'/----------------------------------------------------------
'/  Returns:    Success/Failure  (Boolean)
'/----------------------------------------------------------
'/  Date        Who             What
'
'/----------------------------------------------------------

   RunTimeError ModuleName, FunctionName, ErrorNumber, ErrorDescription, "", "", 0, 0
    
End Sub

