Attribute VB_Name = "basFile"
Option Explicit

'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose       File Utilities Module
'
'               ** THIS MODULE IS FULLY REUSABLE!  PLEASE REUSE IT! **
'
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
'Declarations
Private Const msMODULE_NAME      As String = "basFile"

Public Const gnDISKUTIL_BUFFER_SIZE As Integer = 1000

Public Const gnDISKUTIL_FAIL As Integer = 0
Public Const gnDISKUTIL_OK As Integer = 1
Public Const gnDISKUTIL_FOLDER_EXISTS As Integer = 3

Private Type typSecurityAttributes
    nLength As Long
    lpSecurityDescriptor As Long
    bInheritHandle As Long
End Type


'Win API Declares

#If Win32 Then
    Private Declare Function GetTempFileName Lib "kernel32" Alias "GetTempFileNameA" (ByVal lpszPath As String, ByVal lpPrefixString As String, ByVal wUnique As Long, ByVal lpTempFileName As String) As Long
    Private Declare Function GetTempPath Lib "kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long

    ' Win32 Api to create a folder
    Private Declare Function CreateDirectory Lib "kernel32" _
        Alias "CreateDirectoryA" (ByVal lpNewDirectory As String, _
        lpSecurityAttributes As typSecurityAttributes) As Long
    
    ' Win32 api to get name of current folder
    Private Declare Function GetCurrentDirectory Lib "kernel32" _
        Alias "GetCurrentDirectoryA" (ByVal nBufferLength As Long, _
        ByVal lpBuffer As String) As Long
    
    ' win32 api to set current folder to a new name
    Private Declare Function SetCurrentDirectory Lib "kernel32" _
        Alias "SetCurrentDirectoryA" (ByVal lpPathName As String) As Long
#End If

Public Function bDoesFolderExist(strFolderName As String) As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Determines if a folder exists or not
'Inputs         folder name to check

'Returns        True if folder exists, False if it dont!


'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------
' Declarations:
    Dim lngBufferLength As Long
    Dim strBuffer As String
    Dim lngRetVal As Long
    
    Const sPROCEDURE_NAME        As String = "bDoesFolderExist"    ' Error logging
    
    ' Errors
    On Error GoTo bDoesFolderExist_Error
    
    lngBufferLength = gnDISKUTIL_BUFFER_SIZE
    
    ' initialize buffer for getcurrentdirectory storage
    strBuffer = Space$(lngBufferLength)
    
    ' save current folder
    lngRetVal = GetCurrentDirectory(lngBufferLength, strBuffer)
    
    ' try to set this folder name as current folder
    lngRetVal = SetCurrentDirectory(strFolderName)
    
    ' if it set okay then it obviously exists and we dont have to create it
    bDoesFolderExist = IIf(lngRetVal = 0, False, True)
        
    ' go back to original folder
    lngRetVal = SetCurrentDirectory(strBuffer)
    
    

bDoesFolderExist_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function
    
    ' Errors:
bDoesFolderExist_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo bDoesFolderExist_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If
        
End Function

Public Function lMakeNewFolder(strNewFolderName As String) As Long
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Uses WinAPI to create new folder or switch to it if already exists
'Inputs         Name of the folder to create

'Returns        error number as reported by Windows


'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------
' Declarations:
    Dim lngNoSecurity As typSecurityAttributes
    Dim sProblem As String
    
    Const sPROCEDURE_NAME        As String = "lMakeNewFolder"    ' Error logging
    
    ' Errors
    On Error GoTo lMakeNewFolder_Error
    
    ' if it does not already exist make it
    If Not bDoesFolderExist(strNewFolderName) Then
        lMakeNewFolder = CreateDirectory(strNewFolderName, lngNoSecurity)
    Else
        ' if it does exist switch to it
        SetCurrentDirectory (strNewFolderName)
        lMakeNewFolder = gnDISKUTIL_FOLDER_EXISTS
    End If
    
lMakeNewFolder_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function
    
    ' Errors:
lMakeNewFolder_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo lMakeNewFolder_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If
    
End Function



Function sAddBackSlash(ByVal sDirPath As String) As String
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'/----------------------------------------------------------
'/  This procedure will add a backslash to a directory path.
'/  If the trailing backslash is already there it will detect it.
'/----------------------------------------------------------
'/  Parameters: Directory Path  (String)
'/----------------------------------------------------------
'/  Returns:    Formatted Path  (String)
'/----------------------------------------------------------
'/  Date        Who         What

'/----------------------------------------------------------
 ' Declarations:
Const sPROCEDURE_NAME       As String = "sAddBackSlash"    ' Error logging
 ' Errors
On Error GoTo sAddBackSlash_Error

  sDirPath = Trim$(sDirPath)
    If sDirPath <> "" Then
        If Right$(sDirPath, 1) <> "\" Then
            sDirPath = sDirPath + "\"
        End If
    End If
    sAddBackSlash = sDirPath

sAddBackSlash_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sAddBackSlash_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sAddBackSlash_Exit
    End If
    
End Function

Public Function sExtractPath(ByVal sFullPathName) As String
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Gets the path only from the full path string
'Returns       The path without the filename
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------
Const sPROCEDURE_NAME       As String = "sExtractPath"    ' Error logging
Dim nPos As Integer
Dim nTotalChars As Integer

' Errors
On Error GoTo sExtractPath_Error
    
    sFullPathName = Trim$(sFullPathName)

    'Check for empty string and store the length of the string
    nTotalChars = Len(sFullPathName)
    If nTotalChars = 0 Then Exit Function

    'Check to see if the string is a path by checking for a backslash
    If InStr(1, sFullPathName, "\") < 1 Then Exit Function

    'Start at end of the string and search each character for a backslash
    For nPos = nTotalChars To 1 Step -1
        If Mid$(sFullPathName, nPos, 1) = "\" Then
            sExtractPath = Left$(sFullPathName, nPos - 1)
            Exit Function
        End If
    Next
    
sExtractPath_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sExtractPath_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sExtractPath_Exit
    End If

End Function



Function bDirectoryExists(ByVal sDirPath As String) As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose            This procedure will check if a directory exists.
'Parameters:    Directory Path  (String)
'Returns:       Success/Failure (Boolean)
'----------------------------------------------------------
'  Date        Who         What
'
'----------------------------------------------------------
  ' Declarations:
Const sPROCEDURE_NAME       As String = "bDirectoryExists"    ' Error logging

' Errors
On Error GoTo bDirectoryExists_Error
    
    bDirectoryExists = False
    
    If Len(Dir$(sDirPath, vbDirectory)) <> 0 Then
        bDirectoryExists = True
    End If
    
bDirectoryExists_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
bDirectoryExists_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo bDirectoryExists_Exit
    End If

End Function
Function bFileExists(ByVal psPathFile As String) As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose            This procedure will check if a file exists.
'Parameters:    File Path  (String)
'Returns:       Success/Failure (Boolean)
'----------------------------------------------------------
'  Date        Who         What
'
'----------------------------------------------------------
 ' Declarations:
Const sPROCEDURE_NAME       As String = "bFileExists"    ' Error logging

' Errors
On Error GoTo bFileExists_Error
    
    bFileExists = False
    
    If Len(Dir$(psPathFile, vbNormal)) <> 0 Then
        bFileExists = True
    End If
    
bFileExists_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
bFileExists_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo bFileExists_Exit
    End If
End Function

Public Function sGetTempFile(Optional ByVal sPrefix As String = "tmp", Optional ByVal sPath As String = "") As String
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Returns the name of a tempory file that can be used by an application.
'Inputs        Optional sPrefix - The prefix to be used in the first part of the filename. If
'              ommited the default will be used.
'              Optional sPath - The path that the temporary file is to be created. If ommited then
'              the System TEMP environment variable will be used.
'Assumes
'Returns       The full path and filename of the temporary file.
'Effects
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------
'----------------------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sGetTempFile"    ' Error logging
Const MAX_PATH As Long = 260

Dim lRandomKey As Long
Dim sBuffer As String


' Errors
On Error GoTo sGetTempFile_Error

    
    'Validate the arguments and set the defaults
    If Len(Trim$(sPrefix)) = 0 Then sPrefix = "tmp"
    If Len(Trim$(sPath)) = 0 Then sPath = sGetTempPathName()
    
    'Fill the buffer
    sBuffer = Space$(MAX_PATH)
    
    'Get the temporary file
    lRandomKey = GetTempFileName(sPath, sPrefix, 0&, sBuffer)
    If lRandomKey = 0 Then Exit Function
    
    'Get rid of the null character
    sGetTempFile = Trim$(Left$(sBuffer, InStr(sBuffer, vbNullChar) - 1))

sGetTempFile_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sGetTempFile_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sGetTempFile_Exit
    End If
    
End Function


Public Function sGetTempPathName() As String
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Retrieves the path that is designated for use by tempory files.
'Inputs        None
'Assumes
'Returns       The tempory path specified by the TMP or the TEMP environment variable.
'Effects
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sGetTempPathName"    ' Error logging
'Declare variables here
Const MAX_PATH As Long = 260

Dim lBufferLength As Long
Dim sBuffer As String

' Errors
On Error GoTo sGetTempPathName_Error

  
      'Fill the buffer
    sBuffer = Space$(MAX_PATH)
    
    'Get the temp path
    lBufferLength = GetTempPath(MAX_PATH, sBuffer)
    If lBufferLength = 0 Then Exit Function
    
    'Get rid of the null character
    sGetTempPathName = Left$(sBuffer, lBufferLength)

sGetTempPathName_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sGetTempPathName_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sGetTempPathName_Exit
    End If
    
End Function

Public Function sExtractFileName(ByVal sFullPathName As String) As String
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Gets the filename from a full path string
'Returns       The filename
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------

'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "sExtractFileName"    ' Error logging
Dim nPos As Integer
Dim nTotalChars As Integer

' Errors
On Error GoTo sExtractFileName_Error

    
    sFullPathName = Trim$(sFullPathName)

    'Check for empty string and store the length of the string
    nTotalChars = Len(sFullPathName)
    If nTotalChars = 0 Then Exit Function

    'Check to see if the string contains a path by checking for a slash
    If InStr(1, sFullPathName, "\") < 1 Then Exit Function

    'Start at end of the string and search each character for a backslash
    For nPos = nTotalChars To 1 Step -1
        If Mid$(sFullPathName, nPos, 1) = "\" Then
            sExtractFileName = Right$(sFullPathName$, nTotalChars - nPos)
            Exit Function
        End If
    Next

sExtractFileName_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
sExtractFileName_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo sExtractFileName_Exit
    End If

End Function


