Attribute VB_Name = "basForm"
Option Explicit

'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose       Form Utilities Module
'
'               ** THIS MODULE IS FULLY REUSABLE!  PLEASE REUSE IT! **
'
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
'Declarations
Private Const msMODULE_NAME      As String = "basForm"


#If Win32 Then
    Private Declare Function SetWindowPos Lib "user32" _
        (ByVal hwnd As Long, ByVal hWndInsertAfter As Long, ByVal X As Long, ByVal Y As Long, _
         ByVal cx As Long, ByVal cy As Long, ByVal wFlags As Long) As Long
    
#Else
    Private Declare Function SetWindowPos Lib "User" _
        (ByVal hwnd%, ByVal hWindInsertAfter%, ByVal X%, ByVal Y%, _
         ByVal cx%, ByVal cy%, ByVal wFlags%) As Integer
#End If


'Progress Form Declares for API SetWindowPos
Private Const SWP_NOMOVE = 2
Private Const SWP_NOSIZE = 1
Private Const Flags = SWP_NOMOVE Or SWP_NOSIZE
Private Const HWND_TOPMOST = -1
Private Const HWND_NOTOPMOST = -2


Public Sub SetMeTopMost(frmThisForm As Form, ByVal bTopMost As Boolean)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Sets/Unsets a window as being "topmost" (floating window)
'Inputs         frmThisForm - Pointer to a form (window) to use
'                    bTopMost - boolean indicating to float or unfloat the window
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "SetMeTopMost"    ' Error logging

Dim lResult As Long

' Errors
On Error GoTo SetMeTopMost_Error

    'Float or Unfloat the form based on the boolean parameter
    If bTopMost Then
        lResult = SetWindowPos(frmThisForm.hwnd, HWND_TOPMOST, 0, 0, 0, 0, Flags)
    Else
        lResult = SetWindowPos(frmThisForm.hwnd, HWND_NOTOPMOST, 0, 0, 0, 0, Flags)
    End If


SetMeTopMost_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
SetMeTopMost_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo SetMeTopMost_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If
        
End Sub

Public Sub MMAShowModal(frmThisForm As Form, Optional frmOwnerForm As Form = Nothing)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose        Displays a form modally and sets a global variable indicating that a modal form is showing.
'Inputs         frmThisForm - Pointer to a form (window) to display
'               optional string representing who the displayed form belongs to.
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "MMAShowModal"    ' Error logging

' Errors
On Error GoTo MMAShowModal_Error
    
    gbModalWindowActive = True
    'If IsMissing(frmOwnerForm) Then
    If frmOwnerForm Is Nothing Then
        frmThisForm.Show vbModal
    Else
        frmThisForm.Show vbModal, frmOwnerForm
    End If
    gbModalWindowActive = False
    
MMAShowModal_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
MMAShowModal_Error:
    ' Process the error
    With Err
        RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
    End With
    
    If gbDesignTime Then
        Stop
        Resume
    Else
        GoTo MMAShowModal_Exit
        Resume 'Does Nothing - For Debugging Purposes
    End If
        
End Sub


Public Sub CenterForm(frmThisForm As Form)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Centers a form on the screen
'Inputs         Pointer to a form
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "CenterForm"    ' Error logging

' Errors
On Error GoTo CenterForm_Error

    
    With frmThisForm
       .Move (Screen.Width - .Width) / 2, (Screen.Height - .Height) / 2
    End With


CenterForm_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
CenterForm_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo CenterForm_Exit
    End If

End Sub

Public Sub CenterChild(frmParent As Form, frmChild As Form, Optional bIgnoreToolBar As Boolean = False)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Centers a child form relative to its parent
'Inputs         Pointer to the parent form.  Pointer to the child form.  Optional boolean to incorporate a standard toolbar height
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "CenterChild"    ' Error logging
Dim nTop As Integer, nLeft As Integer
Dim nParentHeight As Integer, nParentWidth As Integer

' Errors
On Error GoTo CenterChild_Error

    
    With frmParent
'        If Not bIgnoreToolBar Then
'            nParentHeight = .sbrMain.Top - (.tbrMain.Top + .tbrMain.Height)
'        Else
            nParentHeight = .Height '- 1000
'        End If
        nParentWidth = .Width - 150
    End With
    
    With frmChild
        nLeft = (nParentWidth - .Width) / 2
        nLeft = IIf(nLeft < 0, 0, nLeft)
        nTop = (nParentHeight - .Height) / 2
        nTop = IIf(nTop < 0, 0, nTop)
        .Move nLeft, nTop
    End With


CenterChild_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
CenterChild_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo CenterChild_Exit
    End If
        
    
End Sub
Public Sub MDICenterChild(frmParent As Form, frmChild As Form, Optional bExcludeToolbarWidth As Boolean = False)
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'Purpose      Centers a child form relative to its MDI Parent
'Inputs         Pointer to the parent form.  Pointer to the child form.  Optional boolean to incorporate a standard toolbar height
'------------------------------------------------------------------------------------------------------
'   Revision History:
'   Date:       Developer:                  Description:
'------------------------------------------------------------------------------------------------------
'
'------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "MDICenterChild"    ' Error logging
Const nMDI_MENU_TOOLBAR_WIDTH As Integer = 1425
Dim lLeftPos As Long, lTopPos As Long, lChildBottom As Long, lMaxValidTop As Long, lStatusBarTop As Long

' Errors
On Error GoTo MDICenterChild_Error

    'Center the form over the MDI child calling it
    lLeftPos = frmParent.Left + ((frmParent.ScaleWidth - frmChild.Width) / 2) + 25
    lTopPos = frmParent.Top + ((frmParent.Height - frmChild.Height) / 2) + IIf(Not IsMissing(bExcludeToolbarWidth), 0, nMDI_MENU_TOOLBAR_WIDTH)
    
    If Not bExcludeToolbarWidth Then
        lStatusBarTop = 1150
    Else
        lStatusBarTop = 1000
    End If
    lMaxValidTop = lStatusBarTop - frmChild.Height
    If lTopPos > lMaxValidTop Then
        frmChild.Move lLeftPos, lMaxValidTop
    Else
        frmChild.Move lLeftPos, lTopPos
    End If



MDICenterChild_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Sub

' Errors:
MDICenterChild_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo MDICenterChild_Exit
    End If
    
End Sub

Function bFormLoaded(ByVal pstrForm As String) As Boolean
'/----------------------------------------------------------
'/  Returns True if ths form is already loaded
'/----------------------------------------------------------
'/  Parameters: None
'/----------------------------------------------------------
'/  Returns:
'/----------------------------------------------------------
'/  Date        Who         What
'
'/----------------------------------------------------------
    Dim objForm As Form
    Dim sCurForm As String
    
    On Error GoTo bFormLoadedErr
    
    bFormLoaded = False
    
    pstrForm = Trim$(UCase$(pstrForm))
    
    For Each objForm In Forms
        sCurForm = Trim$(UCase$(objForm.Name))
        If StrComp(pstrForm, sCurForm, vbTextCompare) = 0 Then
            bFormLoaded = True
            Exit For
        End If
    Next
    
            
bFormLoadedExit:
    On Error Resume Next
    '/ Perform all necessary cleanup work here
    Set objForm = Nothing
    Exit Function
bFormLoadedErr:
    On Error Resume Next
    Err.Clear
    GoTo bFormLoadedExit
    
End Function

Function nMDIWindowOffset() As Integer
''------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'/----------------------------------------------------------
'/  This procedure will return the offset position to display
'/  the next MDI child window, based on the number of open windows
'/----------------------------------------------------------
'/  Parameters: None
'/----------------------------------------------------------
'/  Returns:    Offset Amount   (Integer)
'/----------------------------------------------------------
'/  Date        Who         What
'/
'/----------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "nMDIWindowOffset"    ' Error logging

' Errors
On Error GoTo nMDIWindowOffset_Error
    
    nMDIWindowOffset = (Forms.Count - 2) * 330

nMDIWindowOffset_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
nMDIWindowOffset_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo nMDIWindowOffset_Exit
    End If
    
End Function

Public Function bControlArrayEnable(ByVal vControlArray As Variant, ByVal bEnabled As Boolean) As Boolean
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Enables or disables all items in a control array
'Inputs        vControlArray - Control Array
'              bEnabled - True to enable
'                         False to disable
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "bControlArrayEnable"    ' Error logging

' Errors
On Error GoTo bControlArrayEnable_Error


    'Declare variables here
    Dim ctl As Control
    
     For Each ctl In vControlArray
        ctl.Enabled = bEnabled
    Next

    bControlArrayEnable = True
    
bControlArrayEnable_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
bControlArrayEnable_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo bControlArrayEnable_Exit
    End If
End Function



Public Function nGetOptionArrayIndexTrue(ByVal vOptionArrayName As Variant) As Integer
'----------------------------------------------------------------------------------------------------
'Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic, Inc.
'----------------------------------------------------------------------------------------------------
'Section       Comment Description
'
'Purpose       Gets the index of the Option Button Control Array that is true
'Returns       The Index of the Control Array that the value is true
'----------------------------------------------------------------------------------------------------
'Revision History:
'Date          Developer                     Description
'----------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------
' Declarations:
Const sPROCEDURE_NAME       As String = "nGetOptionArrayIndexTrue"    ' Error logging
Dim optButton As OptionButton

' Errors
On Error GoTo nGetOptionArrayIndexTrue_Error


    'Set return value to -1 because 0 could be the first index of a control array
    nGetOptionArrayIndexTrue = -1
    
    For Each optButton In vOptionArrayName
        If optButton.Value = True Then
            nGetOptionArrayIndexTrue = optButton.Index
            Exit For
        End If
    Next
    

nGetOptionArrayIndexTrue_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
nGetOptionArrayIndexTrue_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo nGetOptionArrayIndexTrue_Exit
    End If

End Function

Function bItemOpen(ByVal sFormName As String, ByVal lID As Long) As Boolean
'------------------------------------------------------------------------------------------------------
'  Developing Applications using Outlook 2000, CDO, Exchange, and Visual Basic
'------------------------------------------------------------------------------------------------------
'Section        Comment Description
'
'/----------------------------------------------------------
'/  This procedure will test if this item is already open in another window
'/----------------------------------------------------------
'/  Parameters: None
'/----------------------------------------------------------
'/  Returns:
'/                  Form Name       (String)
'/                  Record ID         (Long)
'/----------------------------------------------------------
'/  Date        Who         What
'/
'/----------------------------------------------------------
 ' Declarations:
Const sPROCEDURE_NAME       As String = "bItemOpen"    ' Error logging
Dim pForm As Form
Dim nFormCount As Integer
Dim nIndex As Integer

' Errors
On Error GoTo bItemOpen_Error
      
    bItemOpen = False
    
    nFormCount = Forms.Count - 1
    
    For nIndex = 0 To nFormCount
        If sFormName = Forms(nIndex).Name Then
            Set pForm = Forms(nIndex)
            If pForm.ID = lID Then
                pForm.ZOrder vbBringToFront
                'MsgBox "This record is already open in another window.", vbOKOnly + vbInformation, "Load Folder"
                bItemOpen = True
            End If
        End If
    Next nIndex
        
bItemOpen_Exit:
    On Error Resume Next
    'Put cleanup code here
    Exit Function

' Errors:
bItemOpen_Error:
    ' Process the error
    With Err
            RunTimeError msMODULE_NAME, sPROCEDURE_NAME, .Number, .Description, .Source, .HelpFile, .HelpContext, .LastDllError
        End With
        
        If gbDesignTime Then
            Stop
            Resume
        Else
            GoTo bItemOpen_Exit
    End If
End Function


